@@ -9,10 +9,10 @@
 // You should have received a copy of the MIT License along with this software.
 // If not, see <https://opensource.org/licenses/MIT>.
 
-use alure::{Reg32, RegA};
+use aluvm::reg::{Reg32, RegA};
 
 /// Example extension set of operations which are required for RGB
-// TODO: Move to RGB Core Library
+// TODO(#3) Move to RGB Core Library
 pub enum RgbOp {
     /// Counts number of metatdata of specific type
     CountMeta(u16, RegA, Reg32),
