@@ -13,7 +13,7 @@
 //! number representation.
 
 use alloc::format;
-use alloc::string::String;
+use alloc::string::{String, ToString};
 use core::fmt::{
     self, Debug, Display, Formatter, LowerExp, LowerHex, Octal, UpperExp, UpperHex, Write,
 };
@@ -25,7 +25,7 @@ use core::str::FromStr;
 
 use amplify::num::{u1024, u256, u512};
 use half::bf16;
-use rustc_apfloat::{ieee, Float};
+use rustc_apfloat::{ieee, Float, Status};
 
 /// Trait of different number layouts
 pub trait NumberLayout: Copy {
@@ -216,6 +216,16 @@ impl NumberLayout for IntLayout {
     fn sign_byte(self) -> u16 { self.bytes() - 1 }
 }
 
+impl From<IntLayout> for Layout {
+    #[inline]
+    fn from(layout: IntLayout) -> Self { Layout::Integer(layout) }
+}
+
+impl From<&IntLayout> for Layout {
+    #[inline]
+    fn from(layout: &IntLayout) -> Self { Layout::Integer(*layout) }
+}
+
 /// Layout of the float value encoding.
 ///
 /// Defines bit dimensionality and encoding format for float types.
@@ -279,6 +289,21 @@ impl NumberLayout for FloatLayout {
 }
 
 impl FloatLayout {
+    /// Constructs [`FloatLayout`] from byte representation
+    pub fn with(value: u8) -> Option<Self> {
+        Some(match value {
+            x if x == FloatLayout::BFloat16 as u8 => FloatLayout::BFloat16,
+            x if x == FloatLayout::IeeeHalf as u8 => FloatLayout::IeeeHalf,
+            x if x == FloatLayout::IeeeSingle as u8 => FloatLayout::IeeeSingle,
+            x if x == FloatLayout::IeeeDouble as u8 => FloatLayout::IeeeDouble,
+            x if x == FloatLayout::IeeeQuad as u8 => FloatLayout::IeeeQuad,
+            x if x == FloatLayout::IeeeOct as u8 => FloatLayout::IeeeOct,
+            x if x == FloatLayout::X87DoubleExt as u8 => FloatLayout::X87DoubleExt,
+            x if x == FloatLayout::FloatTapered as u8 => FloatLayout::FloatTapered,
+            _ => return None,
+        })
+    }
+
     /// Detects if layout is used for encoding floating-point numbers
     #[inline]
     pub fn is_float(self) -> bool { self as u8 > 1 }
@@ -319,8 +344,18 @@ impl FloatLayout {
     }
 }
 
+impl From<FloatLayout> for Layout {
+    #[inline]
+    fn from(layout: FloatLayout) -> Self { Layout::Float(layout) }
+}
+
+impl From<&FloatLayout> for Layout {
+    #[inline]
+    fn from(layout: &FloatLayout) -> Self { Layout::Float(*layout) }
+}
+
 /// Representation of the value from a register, which may be `None` if the register is unset.
-#[derive(Copy, Clone, PartialEq, Eq, Hash, Debug, Default, From)]
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Default, From)]
 pub struct MaybeNumber(Option<Number>);
 
 impl MaybeNumber {
@@ -708,7 +743,63 @@ impl Number {
                 self.clean();
                 bit_len <= len2 * 8
             }
-            (from, to) => unimplemented!("Number layout reshape from {} to {}", from, to),
+            (Layout::Float(l1), Layout::Float(l2)) => {
+                let value = match l1 {
+                    FloatLayout::BFloat16 => bf16::from(*self).to_string(),
+                    FloatLayout::IeeeHalf => ieee::Half::from(*self).to_string(),
+                    FloatLayout::IeeeSingle => ieee::Single::from(*self).to_string(),
+                    FloatLayout::IeeeDouble => ieee::Double::from(*self).to_string(),
+                    FloatLayout::X87DoubleExt => ieee::X87DoubleExtended::from(*self).to_string(),
+                    FloatLayout::IeeeQuad => ieee::Quad::from(*self).to_string(),
+                    FloatLayout::IeeeOct => {
+                        unimplemented!("IEEE octal precision layout conversion")
+                    }
+                    FloatLayout::FloatTapered => unimplemented!("tapered float layout conversion"),
+                };
+                *self = match l2 {
+                    FloatLayout::BFloat16 => {
+                        bf16::from_str(&value).map(Number::from).expect("float layout conversion")
+                    }
+                    FloatLayout::IeeeHalf => ieee::Half::from_str(&value)
+                        .map(Number::from)
+                        .expect("float layout conversion"),
+                    FloatLayout::IeeeSingle => ieee::Single::from_str(&value)
+                        .map(Number::from)
+                        .expect("float layout conversion"),
+                    FloatLayout::IeeeDouble => ieee::Double::from_str(&value)
+                        .map(Number::from)
+                        .expect("float layout conversion"),
+                    FloatLayout::X87DoubleExt => ieee::X87DoubleExtended::from_str(&value)
+                        .map(Number::from)
+                        .expect("float layout conversion"),
+                    FloatLayout::IeeeQuad => ieee::Quad::from_str(&value)
+                        .map(Number::from)
+                        .expect("float layout conversion"),
+                    FloatLayout::IeeeOct => {
+                        unimplemented!("IEEE octal precision layout conversion")
+                    }
+                    FloatLayout::FloatTapered => unimplemented!("tapered float layout conversion"),
+                };
+                false
+            }
+            (Layout::Float(fl), Layout::Integer(_)) => {
+                let val = match fl {
+                    FloatLayout::BFloat16 => todo!("BFloat16 to integer conversion"),
+                    FloatLayout::IeeeHalf => ieee::Half::from(*self).to_i128(128),
+                    FloatLayout::IeeeSingle => ieee::Single::from(*self).to_i128(128),
+                    FloatLayout::IeeeDouble => ieee::Double::from(*self).to_i128(128),
+                    FloatLayout::X87DoubleExt => ieee::X87DoubleExtended::from(*self).to_i128(128),
+                    FloatLayout::IeeeQuad => ieee::Quad::from(*self).to_i128(128),
+                    FloatLayout::IeeeOct => {
+                        unimplemented!("IEEE octal precision layout conversion")
+                    }
+                    FloatLayout::FloatTapered => unimplemented!("tapered float layout conversion"),
+                };
+                *self = Number::from(val.value);
+                self.reshape(to);
+                val.status == Status::OK
+            }
+            (from, to) => todo!("Number layout reshape from {} to {}", from, to),
         }
     }
 
@@ -799,7 +890,10 @@ impl Debug for Number {
 impl Display for Number {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         match self.layout {
-            Layout::Integer(IntLayout { signed: false, .. }) if self.min_bit_len() > 12 => {
+            Layout::Integer(IntLayout { signed: false, .. }) if self.min_bit_len() <= 12 => {
+                write!(f, "{}", u16::from(self))
+            }
+            Layout::Integer(IntLayout { signed: false, .. }) if self.min_bit_len() < 16 * 8 => {
                 write!(f, "0x{:X}", self)
             }
             Layout::Integer(IntLayout { signed: true, bytes }) if bytes <= 16 => {
@@ -808,7 +902,7 @@ impl Display for Number {
             Layout::Integer(IntLayout { signed: false, bytes }) if bytes <= 16 => {
                 Display::fmt(&u128::from(self), f)
             }
-            Layout::Integer(IntLayout { signed: false, .. }) if self.min_bit_len() < 256 => {
+            Layout::Integer(IntLayout { signed: false, bytes }) if bytes <= 32 => {
                 Display::fmt(&u256::from(self), f)
             }
             Layout::Integer(IntLayout { signed: false, .. }) if self.min_bit_len() < 512 => {
@@ -969,9 +1063,9 @@ macro_rules! impl_number_bytes_conv {
     ($len:literal) => {
         impl From<Number> for [u8; $len] {
             fn from(val: Number) -> Self {
-                let len = val.len() as usize;
+                let len = val.min_bit_len() as usize / 8;
                 assert!(
-                    val.len() <= $len,
+                    len <= $len,
                     "attempt to convert number into a byte array with incorrect length",
                 );
                 let mut bytes = [0u8; $len];
@@ -1011,7 +1105,7 @@ macro_rules! impl_number_int_conv {
         impl From<Number> for $ty {
             fn from(val: Number) -> Self {
                 assert!(
-                    val.layout.bytes() as u32 <= $len,
+                    val.min_bit_len() <= $len * 8,
                     "attempt to convert Number into type with lower bit dimension"
                 );
                 $ty::from_le_bytes(<[u8; $len]>::from(val))
@@ -1036,7 +1130,7 @@ macro_rules! impl_number_float_conv {
         impl From<Number> for $ty {
             fn from(val: Number) -> Self {
                 assert!(
-                    val.layout.bytes() as u32 <= $len,
+                    val.min_bit_len() <= $len * 8,
                     "attempt to convert Number into type with lower bit dimension"
                 );
                 $ty::from_bits(val.into())
