@@ -23,9 +23,9 @@ use super::{
     Instr, MoveOp, PutOp, ReservedOp, Secp256k1Op,
 };
 use crate::data::{ByteStr, MaybeNumber, Number, NumberLayout};
-use crate::isa::{FloatEqFlag, IntFlags, MergeFlag, SignFlag};
+use crate::isa::{ExtendFlag, FloatEqFlag, IntFlags, MergeFlag, NoneEqFlag, SignFlag};
 use crate::libs::{constants, LibSite};
-use crate::reg::{CoreRegs, Reg32, RegA, RegR, RegisterFamily};
+use crate::reg::{CoreRegs, NumericRegister, Reg32, RegA, RegR};
 
 /// Turing machine movement after instruction execution
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug)]
@@ -64,11 +64,12 @@ pub trait InstructionSet: Bytecode + core::fmt::Display + core::fmt::Debug {
     fn isa_id() -> Box<[u8]> { Self::isa_string().as_bytes().into() }
 
     /// Checks whether provided ISA extension ID is supported by the current instruction set
-    fn is_supported(id: &str) -> bool { id == constants::ISA_ID_ALU }
+    #[inline]
+    fn is_supported(id: &str) -> bool { Self::isa_ids().contains(id) }
 
     /// Returns computational complexity of the instruction
     #[inline]
-    fn complexity(&self) -> u32 { 1 }
+    fn complexity(&self) -> u64 { 1 }
 
     /// Executes given instruction taking all registers as input and output.
     ///
@@ -98,14 +99,6 @@ where
         set
     }
 
-    fn is_supported(id: &str) -> bool {
-        id == constants::ISA_ID_ALU
-            || DigestOp::is_supported(id)
-            || Secp256k1Op::is_supported(id)
-            || Curve25519Op::is_supported(id)
-            || Extension::is_supported(id)
-    }
-
     #[inline]
     fn exec(&self, regs: &mut CoreRegs, site: LibSite) -> ExecStep {
         match self {
@@ -133,7 +126,7 @@ impl InstructionSet for ControlFlowOp {
     fn isa_ids() -> BTreeSet<&'static str> { BTreeSet::default() }
 
     #[inline]
-    fn complexity(&self) -> u32 { 2 }
+    fn complexity(&self) -> u64 { 2 }
 
     fn exec(&self, regs: &mut CoreRegs, site: LibSite) -> ExecStep {
         match self {
@@ -174,7 +167,7 @@ impl InstructionSet for PutOp {
     fn isa_ids() -> BTreeSet<&'static str> { BTreeSet::default() }
 
     #[inline]
-    fn complexity(&self) -> u32 { 2 }
+    fn complexity(&self) -> u64 { 2 }
 
     fn exec(&self, regs: &mut CoreRegs, _: LibSite) -> ExecStep {
         match self {
@@ -343,7 +336,7 @@ impl InstructionSet for CmpOp {
                 regs.st0 = regs
                     .get_both(reg, idx1, reg, idx2)
                     .map(|(val1, val2)| val1 == val2)
-                    .unwrap_or(*st);
+                    .unwrap_or(*st == NoneEqFlag::Equal);
             }
             CmpOp::EqF(eq_flag, reg, idx1, idx2) => {
                 regs.st0 = regs
@@ -361,7 +354,7 @@ impl InstructionSet for CmpOp {
                 regs.st0 = regs
                     .get_both(reg, idx1, reg, idx2)
                     .map(|(val1, val2)| val1 == val2)
-                    .unwrap_or(*st);
+                    .unwrap_or(*st == NoneEqFlag::Equal);
             }
             CmpOp::IfZA(reg, idx) => {
                 regs.st0 = regs.get(reg, idx).map(Number::is_zero).unwrap_or(false)
@@ -396,7 +389,7 @@ impl InstructionSet for ArithmeticOp {
     fn isa_ids() -> BTreeSet<&'static str> { BTreeSet::default() }
 
     #[inline]
-    fn complexity(&self) -> u32 {
+    fn complexity(&self) -> u64 {
         match self {
             ArithmeticOp::AddF(_, _, _, _)
             | ArithmeticOp::SubF(_, _, _, _)
@@ -547,36 +540,45 @@ impl InstructionSet for BytesOp {
     fn isa_ids() -> BTreeSet<&'static str> { BTreeSet::default() }
 
     #[inline]
-    fn complexity(&self) -> u32 { 5 }
+    fn complexity(&self) -> u64 { 5 }
 
     #[allow(warnings)]
     fn exec(&self, regs: &mut CoreRegs, _site: LibSite) -> ExecStep {
         match self {
             BytesOp::Put(reg, bytes, st0) => {
-                regs.s16.insert(*reg, *bytes.clone());
+                regs.s16[reg.as_usize()] = Some(*bytes.clone());
                 if *st0 {
                     regs.st0 = false
                 }
             }
             BytesOp::Mov(reg1, reg2) => {
-                regs.s16.remove(&reg1).and_then(|bs| regs.s16.insert(*reg2, bs));
+                let bs = regs.s16[reg1.as_usize()].clone();
+                regs.s16[reg1.as_usize()] = None;
+                regs.s16[reg2.as_usize()] = bs;
             }
             BytesOp::Swp(reg1, reg2) => {
-                let bs1 = regs.s16.remove(&reg1);
-                regs.s16.remove(&reg1).and_then(|bs2| regs.s16.insert(*reg1, bs2));
-                bs1.and_then(|bs1| regs.s16.insert(*reg2, bs1));
+                let bs1 = regs.s16[reg1.as_usize()].clone();
+                let bs2 = regs.s16[reg2.as_usize()].clone();
+                regs.s16[reg1.as_usize()] = bs2;
+                regs.s16[reg2.as_usize()] = bs1;
             }
             BytesOp::Fill(reg, offset1, offset2, value, flag) => {
                 let mut f = || -> Option<()> {
                     let o1 = regs.a16[offset1.to_usize()]?;
                     let o2 = regs.a16[offset2.to_usize()]?;
-                    let range = o1..=o2;
+                    let range = o1..o2;
                     let val = regs.a8[value.to_usize()]?;
-                    let bs = regs.s16.entry(*reg).or_insert(ByteStr::default());
-                    if bs.len() <= *range.end() as usize && !flag {
+                    let ref mut bs = regs.s16[reg.as_usize()];
+                    let bs = if let Some(s) = bs {
+                        s
+                    } else {
+                        *bs = Some(ByteStr::default());
+                        bs.as_mut().expect("rust optionals are broken")
+                    };
+                    if bs.len() <= range.end && *flag == ExtendFlag::Fail {
                         return None;
                     }
-                    bs.fill(o1..=o2, val);
+                    bs.fill(o1..o2, val);
                     Some(())
                 };
                 f().unwrap_or_else(|| regs.st0 = false);
@@ -585,7 +587,7 @@ impl InstructionSet for BytesOp {
                 let mut f = || -> Option<()> {
                     let s = regs.get_s(*src)?;
                     let len = s.len();
-                    if !reg.int_layout().fits_usize(len) {
+                    if !reg.int_layout().fits_usize(len as usize) {
                         return None;
                     }
                     regs.set(reg, dst, len as u32);
@@ -599,7 +601,7 @@ impl InstructionSet for BytesOp {
             BytesOp::Cnt(src, byte, dst) => {
                 let mut f = || -> Option<()> {
                     let val = regs.a8[*byte as u8 as usize]?;
-                    let bs = regs.s16.get(&src)?;
+                    let bs = regs.s16[src.as_usize()].as_ref()?;
                     let count = bs.as_ref().into_iter().filter(|b| **b == val).count();
                     if !RegA::A16.int_layout().fits_usize(count) {
                         return None;
@@ -624,8 +626,8 @@ impl InstructionSet for BytesOp {
             BytesOp::Find(reg1, reg2) => {
                 let mut f = || -> Option<()> {
                     let (s1, s2) = regs.get_both_s(*reg1, *reg2)?;
-                    let r1 = s1.bytes.as_ref();
-                    let r2 = s2.bytes.as_ref();
+                    let r1 = s1.as_ref();
+                    let r2 = s2.as_ref();
                     let len = r2.len();
                     let mut count = 0usize;
                     for i in 0..r1.len() {
@@ -650,7 +652,7 @@ impl InstructionSet for BytesOp {
                     let mut s = regs.get_s(*reg1)?.clone();
                     let bs = s.as_mut();
                     bs.reverse();
-                    regs.s16.insert(*reg2, s);
+                    regs.s16[reg2.as_usize()] = Some(s);
                     Some(())
                 };
                 f().unwrap_or_else(|| {
@@ -663,7 +665,7 @@ impl InstructionSet for BytesOp {
             }
             BytesOp::Extr(src, dst, index, offset) => {
                 let mut f = || -> Option<()> {
-                    let s = regs.get_s(src)?.clone();
+                    let s = regs.get_s(*src)?.clone();
                     let offset = regs.a16[*offset as u8 as usize]?;
                     let end = offset.saturating_add(dst.layout().bytes());
                     let num = Number::from_slice(&s.as_ref()[offset as usize..=end as usize]);
@@ -677,13 +679,13 @@ impl InstructionSet for BytesOp {
             }
             BytesOp::Inj(src, dst, index, offset) => {
                 let mut f = || -> Option<()> {
-                    let mut s = regs.get_s(src)?.clone();
+                    let mut s = regs.get_s(*src)?.clone();
                     let val = regs.get(dst, index).map(|v| v)?;
                     let offset = regs.a16[*offset as u8 as usize]?;
                     let end = offset.saturating_add(dst.layout().bytes() - 1);
-                    s.adjust_len(end, true);
+                    s.adjust_len(end);
                     s.as_mut()[offset as usize..=end as usize].copy_from_slice(val.as_ref());
-                    regs.s16.insert(*src as u8, s);
+                    regs.s16[src.as_usize()] = Some(s);
                     Some(())
                 };
                 f().unwrap_or_else(|| {
@@ -694,24 +696,20 @@ impl InstructionSet for BytesOp {
             BytesOp::Join(src1, src2, dst) => {
                 let mut f = || -> Option<()> {
                     let (s1, s2) = regs.get_both_s(*src1, *src2)?;
-                    let len = s1.len() + s2.len();
-                    if len > u16::MAX as usize + 1 {
+                    if s1.len() as usize + s2.len() as usize > u16::MAX as usize {
                         return None;
                     }
+                    let len = s1.len() + s2.len();
                     let mut d = s1.clone();
-                    if len == u16::MAX as usize + 1 {
-                        d.adjust_len((len - 1) as u16, true);
-                    } else {
-                        d.adjust_len(len as u16, false);
-                    }
+                    d.adjust_len(len);
                     let mut d = ByteStr::with(s1);
-                    d.as_mut()[s1.len()..].copy_from_slice(s2.as_ref());
-                    regs.s16.insert(*dst, d);
+                    d.as_mut()[s1.len() as usize..].copy_from_slice(s2.as_ref());
+                    regs.s16[dst.as_usize()] = Some(d);
                     Some(())
                 };
                 f().unwrap_or_else(|| {
                     regs.st0 = false;
-                    regs.s16.remove(&dst);
+                    regs.s16[dst.as_usize()] = None;
                 })
             }
             BytesOp::Splt(flag, offset, src, dst1, dst2) => {
@@ -737,30 +735,27 @@ impl InstructionSet for DigestOp {
     }
 
     #[inline]
-    fn complexity(&self) -> u32 { 100 }
-
-    #[inline]
-    fn is_supported(id: &str) -> bool { id == constants::ISA_ID_BPDIGEST }
+    fn complexity(&self) -> u64 { 100 }
 
     fn exec(&self, regs: &mut CoreRegs, _site: LibSite) -> ExecStep {
         let none;
         match self {
             DigestOp::Ripemd(src, dst) => {
-                let s = regs.get_s(src);
+                let s = regs.get_s(*src);
                 none = s.is_none();
-                let hash = s.map(|s| ripemd160::Hash::hash(&s.bytes[..]).into_inner());
+                let hash = s.map(|s| ripemd160::Hash::hash(s.as_ref()).into_inner());
                 regs.set(RegR::R160, dst, hash);
             }
             DigestOp::Sha256(src, dst) => {
-                let s = regs.get_s(src);
+                let s = regs.get_s(*src);
                 none = s.is_none();
-                let hash = s.map(|s| sha256::Hash::hash(&s.bytes[..]).into_inner());
+                let hash = s.map(|s| sha256::Hash::hash(s.as_ref()).into_inner());
                 regs.set(RegR::R256, dst, hash);
             }
             DigestOp::Sha512(src, dst) => {
-                let s = regs.get_s(src);
+                let s = regs.get_s(*src);
                 none = s.is_none();
-                let hash = s.map(|s| sha512::Hash::hash(&s.bytes[..]).into_inner());
+                let hash = s.map(|s| sha512::Hash::hash(s.as_ref()).into_inner());
                 regs.set(RegR::R512, dst, hash);
             }
         }
@@ -784,16 +779,8 @@ impl InstructionSet for Secp256k1Op {
         set
     }
 
-    #[cfg(feature = "secp256k1")]
     #[inline]
-    fn is_supported(id: &str) -> bool { id == constants::ISA_ID_SECP256K }
-
-    #[inline]
-    fn complexity(&self) -> u32 { 1000 }
-
-    #[cfg(not(feature = "secp256k1"))]
-    #[inline]
-    fn is_supported(_: &str) -> bool { false }
+    fn complexity(&self) -> u64 { 1000 }
 
     #[cfg(not(feature = "secp256k1"))]
     fn exec(&self, _: &mut CoreRegs, _: LibSite) -> ExecStep {
@@ -895,16 +882,8 @@ impl InstructionSet for Curve25519Op {
         set
     }
 
-    #[cfg(feature = "curve25519")]
     #[inline]
-    fn is_supported(id: &str) -> bool { id == constants::ISA_ID_ED25519 }
-
-    #[cfg(not(feature = "curve25519"))]
-    #[inline]
-    fn is_supported(_: &str) -> bool { false }
-
-    #[inline]
-    fn complexity(&self) -> u32 { 1000 }
+    fn complexity(&self) -> u64 { 1000 }
 
     #[cfg(not(feature = "curve25519"))]
     fn exec(&self, _: &mut CoreRegs, _: LibSite) -> ExecStep {
