@@ -19,25 +19,28 @@ use core::str::FromStr;
 use amplify::num::{u1, u2, u3};
 use rustc_apfloat::Round;
 
+/// Marker trait for flag types
+pub trait Flag: FromStr<Err = ParseFlagError> + Default {}
+
 /// Errors for parsing string representation for a flag values
 #[derive(Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
 #[cfg_attr(feature = "std", derive(Error))]
 #[display(doc_comments)]
 pub enum ParseFlagError {
-    /// Unknown `{0}` flag `{1}`
+    /// unknown `{0}` flag `{1}`
     UnknownFlag(/** Flag description */ &'static str, /** Unrecognized flag */ char),
 
-    /// Unknown `{0}` flags `{1}`
+    /// unknown `{0}` flags `{1}`
     UnknownFlags(/** Flag description */ &'static str, /** Unrecognized flags */ String),
 
-    /// Only one of mutually exclusive flags must be specified for {0} (only `{1}` or `{2}`)
+    /// only one of mutually exclusive flags must be specified for {0} (only `{1}` or `{2}`)
     MutuallyExclusiveFlags(
         /** Flag description */ &'static str,
         /** Flag 1 */ char,
         /** Flag 2 */ char,
     ),
 
-    /// Required flag for {0} is absent
+    /// required flag for {0} is absent
     RequiredFlagAbsent(/** Flag description */ &'static str),
 
     /// duplicated flags `{1}` are specified for {0}
@@ -56,11 +59,18 @@ pub enum SignFlag {
     Signed = 1,
 }
 
+impl Flag for SignFlag {}
+
+impl Default for SignFlag {
+    #[inline]
+    fn default() -> Self { Self::Unsigned }
+}
+
 impl FromStr for SignFlag {
     type Err = ParseFlagError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        if !s.is_empty() {
+        if s.is_empty() {
             return Err(ParseFlagError::RequiredFlagAbsent("integer sign"));
         }
         let filtered = s.replace(&['u', 's'][..], "");
@@ -110,6 +120,79 @@ impl From<&SignFlag> for bool {
     fn from(flag: &SignFlag) -> Self { *flag == SignFlag::Signed }
 }
 
+/// Non-equality flag
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
+pub enum NoneEqFlag {
+    /// Two `None` register values are considered equal
+    #[display("e")]
+    Equal = 1,
+
+    /// Two `None` register values are considered non-equal
+    #[display("n")]
+    NonEqual = 0,
+}
+
+impl Flag for NoneEqFlag {}
+
+impl Default for NoneEqFlag {
+    #[inline]
+    fn default() -> Self { Self::Equal }
+}
+
+impl FromStr for NoneEqFlag {
+    type Err = ParseFlagError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        if s.is_empty() {
+            return Err(ParseFlagError::RequiredFlagAbsent("none-equality"));
+        }
+        let filtered = s.replace(&['e', 'n'][..], "");
+        if !filtered.is_empty() {
+            return Err(ParseFlagError::UnknownFlags("none-equality", filtered));
+        }
+        match (s.contains('e'), s.contains('n')) {
+            (true, false) => Ok(NoneEqFlag::Equal),
+            (false, true) => Ok(NoneEqFlag::NonEqual),
+            (true, true) => Err(ParseFlagError::MutuallyExclusiveFlags("none-equality", 'e', 'n')),
+            (false, false) => Err(ParseFlagError::RequiredFlagAbsent("none-equality")),
+        }
+    }
+}
+
+impl NoneEqFlag {
+    /// Constructs none-equality flag from `u1` value (used in bytecode serialization)
+    pub fn from_u1(val: u1) -> NoneEqFlag {
+        match val.as_u8() {
+            v if v == NoneEqFlag::Equal as u8 => NoneEqFlag::Equal,
+            v if v == NoneEqFlag::NonEqual as u8 => NoneEqFlag::NonEqual,
+            _ => unreachable!(),
+        }
+    }
+
+    /// Returns `u1` representation of none-equality flag (used in bytecode serialization).
+    pub fn as_u1(self) -> u1 { u1::with(self as u8) }
+}
+
+impl From<u1> for NoneEqFlag {
+    fn from(val: u1) -> NoneEqFlag { NoneEqFlag::from_u1(val) }
+}
+
+impl From<&NoneEqFlag> for u1 {
+    fn from(flag: &NoneEqFlag) -> u1 { flag.as_u1() }
+}
+
+impl From<NoneEqFlag> for u1 {
+    fn from(flag: NoneEqFlag) -> u1 { flag.as_u1() }
+}
+
+impl From<NoneEqFlag> for bool {
+    fn from(flag: NoneEqFlag) -> Self { flag == NoneEqFlag::Equal }
+}
+
+impl From<&NoneEqFlag> for bool {
+    fn from(flag: &NoneEqFlag) -> Self { *flag == NoneEqFlag::Equal }
+}
+
 /// Float equality flag
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
 pub enum FloatEqFlag {
@@ -125,11 +208,18 @@ pub enum FloatEqFlag {
     Rounding = 1,
 }
 
+impl Flag for FloatEqFlag {}
+
+impl Default for FloatEqFlag {
+    #[inline]
+    fn default() -> Self { Self::Exact }
+}
+
 impl FromStr for FloatEqFlag {
     type Err = ParseFlagError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        if !s.is_empty() {
+        if s.is_empty() {
             return Err(ParseFlagError::RequiredFlagAbsent("float equality"));
         }
         let filtered = s.replace(&['e', 'r'][..], "");
@@ -193,11 +283,18 @@ pub enum RoundingFlag {
     Ceil = 3,
 }
 
+impl Flag for RoundingFlag {}
+
+impl Default for RoundingFlag {
+    #[inline]
+    fn default() -> Self { Self::TowardsNearest }
+}
+
 impl FromStr for RoundingFlag {
     type Err = ParseFlagError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        if !s.is_empty() {
+        if s.is_empty() {
             return Err(ParseFlagError::RequiredFlagAbsent("float rounding"));
         }
 
@@ -279,6 +376,13 @@ pub struct IntFlags {
     pub wrap: bool,
 }
 
+impl Flag for IntFlags {}
+
+impl Default for IntFlags {
+    #[inline]
+    fn default() -> Self { Self { signed: false, wrap: false } }
+}
+
 impl Display for IntFlags {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         if self.signed {
@@ -393,11 +497,18 @@ pub enum MergeFlag {
     Or = 3,
 }
 
+impl Flag for MergeFlag {}
+
+impl Default for MergeFlag {
+    #[inline]
+    fn default() -> Self { Self::Set }
+}
+
 impl FromStr for MergeFlag {
     type Err = ParseFlagError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        if !s.is_empty() {
+        if s.is_empty() {
             return Err(ParseFlagError::RequiredFlagAbsent("merge operation"));
         }
 
@@ -455,6 +566,80 @@ impl From<MergeFlag> for u2 {
     fn from(flag: MergeFlag) -> u2 { flag.as_u2() }
 }
 
+/// Flag for bytestring operations indicating whether the string should be extended to a new length
+/// or the operation should fail (for instance, see `fill` operation).
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
+pub enum ExtendFlag {
+    /// Unsigned integer
+    #[display("e")]
+    Extend = 0,
+
+    /// Signed integer
+    #[display("f")]
+    Fail = 1,
+}
+
+impl Flag for ExtendFlag {}
+
+impl Default for ExtendFlag {
+    #[inline]
+    fn default() -> Self { Self::Extend }
+}
+
+impl FromStr for ExtendFlag {
+    type Err = ParseFlagError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        if s.is_empty() {
+            return Err(ParseFlagError::RequiredFlagAbsent("extension flag"));
+        }
+        let filtered = s.replace(&['e', 'f'][..], "");
+        if !filtered.is_empty() {
+            return Err(ParseFlagError::UnknownFlags("extension flag", filtered));
+        }
+        match (s.contains('e'), s.contains('f')) {
+            (true, false) => Ok(ExtendFlag::Extend),
+            (false, true) => Ok(ExtendFlag::Fail),
+            (true, true) => Err(ParseFlagError::MutuallyExclusiveFlags("extension flag", 'e', 'f')),
+            (false, false) => Err(ParseFlagError::RequiredFlagAbsent("extension flag")),
+        }
+    }
+}
+
+impl ExtendFlag {
+    /// Constructs extension flag from `u1` value (used in bytecode serialization)
+    pub fn from_u1(val: u1) -> ExtendFlag {
+        match val.as_u8() {
+            v if v == ExtendFlag::Extend as u8 => ExtendFlag::Extend,
+            v if v == ExtendFlag::Fail as u8 => ExtendFlag::Fail,
+            _ => unreachable!(),
+        }
+    }
+
+    /// Returns `u1` representation of extension flag (used in bytecode serialization).
+    pub fn as_u1(self) -> u1 { u1::with(self as u8) }
+}
+
+impl From<u1> for ExtendFlag {
+    fn from(val: u1) -> ExtendFlag { ExtendFlag::from_u1(val) }
+}
+
+impl From<&ExtendFlag> for u1 {
+    fn from(flag: &ExtendFlag) -> u1 { flag.as_u1() }
+}
+
+impl From<ExtendFlag> for u1 {
+    fn from(flag: ExtendFlag) -> u1 { flag.as_u1() }
+}
+
+impl From<ExtendFlag> for bool {
+    fn from(flag: ExtendFlag) -> Self { flag == ExtendFlag::Fail }
+}
+
+impl From<&ExtendFlag> for bool {
+    fn from(flag: &ExtendFlag) -> Self { *flag == ExtendFlag::Fail }
+}
+
 /// Flags for bytestring split operation.
 ///
 /// If offset exceeds the length of the string in the register, than the behaviour of
@@ -524,11 +709,18 @@ pub enum SplitFlag {
     ZeroZero = 7,
 }
 
+impl Flag for SplitFlag {}
+
+impl Default for SplitFlag {
+    #[inline]
+    fn default() -> Self { Self::NoneNone }
+}
+
 impl FromStr for SplitFlag {
     type Err = ParseFlagError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        if !s.is_empty() {
+        if s.is_empty() {
             return Err(ParseFlagError::RequiredFlagAbsent("split operation"));
         }
 
@@ -634,11 +826,18 @@ pub enum InsertFlag {
     Shorten = 7,
 }
 
+impl Flag for InsertFlag {}
+
+impl Default for InsertFlag {
+    #[inline]
+    fn default() -> Self { Self::FailOnLen }
+}
+
 impl FromStr for InsertFlag {
     type Err = ParseFlagError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        if !s.is_empty() {
+        if s.is_empty() {
             return Err(ParseFlagError::RequiredFlagAbsent("insert operation"));
         }
         let filtered = s.replace(&['l', 'o', 'f', 'e', 'a', 'x', 'c', 's'][..], "");
@@ -727,11 +926,18 @@ pub enum DeleteFlag {
     Extend = 3,
 }
 
+impl Flag for DeleteFlag {}
+
+impl Default for DeleteFlag {
+    #[inline]
+    fn default() -> Self { Self::None }
+}
+
 impl FromStr for DeleteFlag {
     type Err = ParseFlagError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        if !s.is_empty() {
+        if s.is_empty() {
             return Err(ParseFlagError::RequiredFlagAbsent("delete operation"));
         }
         let filtered = s.replace(&['n', 'z', 'c', 'e'][..], "");
