@@ -11,15 +11,14 @@
 
 use alloc::boxed::Box;
 
-use amplify::num::u6;
-
 use super::{
     DeleteFlag, FloatEqFlag, InsertFlag, InstructionSet, IntFlags, MergeFlag, RoundingFlag,
     SignFlag, SplitFlag,
 };
 use crate::data::{ByteStr, MaybeNumber, Step};
+use crate::isa::{ExtendFlag, NoneEqFlag};
 use crate::libs::LibSite;
-use crate::reg::{Reg16, Reg32, Reg8, RegA, RegA2, RegAF, RegAR, RegBlockAR, RegF, RegR};
+use crate::reg::{Reg16, Reg32, Reg8, RegA, RegA2, RegAF, RegAR, RegBlockAR, RegF, RegR, RegS};
 
 /// Reserved instruction, which equal to [`ControlFlowOp::Fail`].
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display, Default)]
@@ -310,25 +309,25 @@ pub enum CmpOp {
     /// Compares value of two integer arithmetic registers setting `st0` to `true` if the first
     /// parameter is greater (and not equal) than the second one. If at least one of the registers
     /// is set to `None`, sets `st0` to `false`.
-    #[display("gt:{0}    {1}{2},{1}{3}")]
+    #[display("gt.{0}    {1}{2},{1}{3}")]
     GtA(SignFlag, RegA, Reg32, Reg32),
 
     /// Compares value of two integer arithmetic registers setting `st0` to `true` if the first
     /// parameter is lesser (and not equal) than the second one. If at least one of the registers
     /// is set to `None`, sets `st0` to `false`.
-    #[display("lt:{0}    {1}{2},{1}{3}")]
+    #[display("lt.{0}    {1}{2},{1}{3}")]
     LtA(SignFlag, RegA, Reg32, Reg32),
 
     /// Compares value of two float arithmetic registers setting `st0` to `true` if the first
     /// parameter is greater (and not equal) than the second one. If at least one of the registers
     /// is set to `None`, sets `st0` to `false`.
-    #[display("gt:{0}    {1}{2},{1}{3}")]
+    #[display("gt.{0}    {1}{2},{1}{3}")]
     GtF(FloatEqFlag, RegF, Reg32, Reg32),
 
     /// Compares value of two float arithmetic registers setting `st0` to `true` if the first
     /// parameter is lesser (and not equal) than the second one. If at least one of the registers
     /// is set to `None`, sets `st0` to `false`.
-    #[display("lt:{0}    {1}{2},{1}{3}")]
+    #[display("lt.{0}    {1}{2},{1}{3}")]
     LtF(FloatEqFlag, RegF, Reg32, Reg32),
 
     // ----
@@ -345,21 +344,31 @@ pub enum CmpOp {
     LtR(RegR, Reg32, Reg32),
 
     /// Checks equality of value in two integer arithmetic (`A`) registers putting result into
-    /// `st0`.None-equality flag specifies value for `st0` for the cases when both of the registers
-    /// are in `None` state.
-    #[display("eq      {1}{2},{1}{3} .none={0}")]
-    EqA(/** `st0` value if both of the registers are uninitialized */ bool, RegA, Reg32, Reg32),
+    /// `st0`. None-equality flag specifies value for `st0` for the cases when both of the
+    /// registers are in `None` state.
+    #[display("eq.{0}    {1}{2},{1}{3}")]
+    EqA(
+        /** `st0` value if both of the registers are uninitialized */ NoneEqFlag,
+        RegA,
+        Reg32,
+        Reg32,
+    ),
 
     /// Checks equality of value in two float arithmetic (`F`) registers putting result into `st0`.
     /// If both registers are `None`, the `st0` is set to `false`.
-    #[display("eq:{0}    {1}{2},{1}{3}")]
+    #[display("eq.{0}    {1}{2},{1}{3}")]
     EqF(FloatEqFlag, RegF, Reg32, Reg32),
 
     /// Checks equality of value in two non-arithmetic (`R`) registers putting result into `st0`.
     /// None-equality flag specifies value for `st0` for the cases when both of the registers
     /// are in `None` state.
-    #[display("eq      {1}{2},{1}{3} .none={0}")]
-    EqR(/** `st0` value if both of the registers are uninitialized */ bool, RegR, Reg32, Reg32),
+    #[display("eq.{0}    {1}{2},{1}{3}")]
+    EqR(
+        /** `st0` value if both of the registers are uninitialized */ NoneEqFlag,
+        RegR,
+        Reg32,
+        Reg32,
+    ),
 
     // ---
     /// Checks if the value in `A` register is equal to zero, setting `st0` to `true` in this case.
@@ -384,11 +393,11 @@ pub enum CmpOp {
 
     /// Takes value from `st0` and merges into the value of the destination `A` register. The merge
     /// operation is defined by the [`MergeFlag`] argument.
-    #[display("st:{0}    {1}{2}")]
+    #[display("st.{0}    {1}{2}")]
     St(MergeFlag, RegA, Reg8),
 
     /// Inverses value in `st0` register
-    #[display("inv     st0")]
+    #[display("stinv")]
     StInv,
 }
 
@@ -400,27 +409,27 @@ pub enum CmpOp {
 #[derive(Copy, Clone, PartialEq, Eq, Hash, Debug, Display)]
 pub enum ArithmeticOp {
     /// Adds values from two integer arithmetic registers and puts result into destination.
-    #[display("add:{0}  {1}{2},{1}{3}")]
+    #[display("add.{0}  {1}{2},{1}{3}")]
     AddA(IntFlags, RegA, Reg32, Reg32),
 
     /// Adds values from two float arithmetic registers and puts result into destination.
-    #[display("add:{0}   {1}{2},{1}{3}")]
+    #[display("add.{0}   {1}{2},{1}{3}")]
     AddF(RoundingFlag, RegF, Reg32, Reg32),
 
     /// Subtracts values from two integer arithmetic registers and puts result into destination.
-    #[display("sub:{0}  {1}{2},{1}{3}")]
+    #[display("sub.{0}  {1}{2},{1}{3}")]
     SubA(IntFlags, RegA, Reg32, Reg32),
 
     /// Subtracts values from two float arithmetic registers and puts result into destination.
-    #[display("sub:{0}   {1}{2},{1}{3}")]
+    #[display("sub.{0}   {1}{2},{1}{3}")]
     SubF(RoundingFlag, RegF, Reg32, Reg32),
 
     /// Multiplies values from two integer arithmetic registers and puts result into destination.
-    #[display("mul:{0}  {1}{2},{1}{3}")]
+    #[display("mul.{0}  {1}{2},{1}{3}")]
     MulA(IntFlags, RegA, Reg32, Reg32),
 
     /// Multiplies values from two float arithmetic registers and puts result into destination.
-    #[display("mul:{0}   {1}{2},{1}{3}")]
+    #[display("mul.{0}   {1}{2},{1}{3}")]
     MulF(RoundingFlag, RegF, Reg32, Reg32),
 
     /// Divides values from two integer arithmetic registers and puts result into destination.
@@ -432,11 +441,11 @@ pub enum ArithmeticOp {
     /// destination must be set to `0` (true) or to None (false).
     ///
     /// NB: Impossible arithmetic operation 0/0 always sets destination to `None`.
-    #[display("div:{0}  {1}{2},{1}{3}")]
+    #[display("div.{0}  {1}{2},{1}{3}")]
     DivA(IntFlags, RegA, Reg32, Reg32),
 
     /// Divides values from two float arithmetic registers and puts result into destination.
-    #[display("div:{0}   {1}{2},{1}{3}")]
+    #[display("div.{0}   {1}{2},{1}{3}")]
     DivF(RoundingFlag, RegF, Reg32, Reg32),
 
     /// Modulo division.
@@ -499,7 +508,7 @@ pub enum BitwiseOp {
     ///
     /// This, [`BitwiseOp::Shl`] and [`BitwiseOp::ShrR`] operations are encoded with the same
     /// instruction bitcode and differ only in their first two argument bits.
-    #[display("shr:{0}   {1}{2},{3}{4}")]
+    #[display("shr.{0}   {1}{2},{3}{4}")]
     ShrA(
         /** Sign flag */ SignFlag,
         /** Which of `A` registers will have a shift value */ RegA2,
@@ -570,27 +579,27 @@ pub enum BitwiseOp {
 pub enum BytesOp {
     /// Put bytestring into a byte string register
     ///
-    /// Data are kept in the separate data segment, thus when the instruction is pared from the
+    /// Data are kept in the separate data segment, thus when the instruction is parsed from the
     /// code segment it knows only data offset and length. If this offset or length exceeds the
     /// size of the data segment, the instruction truncates the string to the part that is present
     /// in the data segment (or zero-length string if the offset exceeds data segment length) and
     /// sets `st0` to `false`. Otherwise, `st0` is unaffected.
-    #[display("put     s16[{0}],{1}")]
+    #[display("put     {0},{1}")]
     Put(
-        /** Destination `s` register index */ u8,
+        /** Destination `s` register index */ RegS,
         Box<ByteStr>,
         /** Indicates that the operation must set `st0` to false; i.e. string data are not
-         * complete */
+         * completely read from the data segment */
         bool,
     ),
 
     /// Move bytestring value between registers
-    #[display("mov     s16[{0}],s16[{1}]")]
-    Mov(/** Source `s` register index */ u8, /** Destination `s` register index */ u8),
+    #[display("mov     {0},s16[{1}]")]
+    Mov(/** Source `s` register index */ RegS, /** Destination `s` register index */ RegS),
 
     /// Swap bytestring value between registers
-    #[display("swp     s16[{0}],s16[{1}]")]
-    Swp(/** First `s` register index */ u8, /** Second `s` register index */ u8),
+    #[display("swp     {0},s16[{1}]")]
+    Swp(/** First `s` register index */ RegS, /** Second `s` register index */ RegS),
 
     /// Fill segment of bytestring with specific byte value, setting the length of the string in
     /// the destination register to specific value.
@@ -610,21 +619,21 @@ pub enum BytesOp {
     ///
     /// If any of the offsets or value registers are unset, sets `st0` to `false` and does not
     /// change destination value.
-    #[display("fill    s16[{0}],{1}..{2},{3}")]
+    #[display("fill    {0},a16{1},a16{2},a8{3}")]
     Fill(
-        /** `s` register index */ u8,
+        /** `s` register index */ RegS,
         /** `a16` register holding first offset */ Reg32,
         /** `a16` register holding second offset (inclusive) */ Reg32,
         /** `a8` register index holding the value */ Reg32,
-        /** Exception handling flag */ bool,
+        /** Exception handling flag */ ExtendFlag,
     ),
 
     /// Put length of the string into the destination register.
     ///
     /// If the string register is empty, or destination register can't fit the length, sets `st0`
     /// to `false` and destination register to `None`.
-    #[display("len     s16[{0}],a16[0]")]
-    Len(/** `s` register index */ u8, RegA, Reg32),
+    #[display("len     {0},a16[0]")]
+    Len(/** `s` register index */ RegS, RegA, Reg32),
 
     /// Count number of byte occurrences from the `a8` register within the string and stores that
     /// value into destination `a16` register.
@@ -634,9 +643,9 @@ pub enum BytesOp {
     ///
     /// If the source byte value register is uninitialized, sets destination register to `None` and
     /// `st0` to `false`.
-    #[display("cnt     s16[{0}],{1},a16[0]")]
+    #[display("cnt     {0},a8{1},a16{2}")]
     Cnt(
-        /** `s` register index */ u8,
+        /** `s` register index */ RegS,
         /** `a8` register with the byte value */ Reg16,
         /** `a16` destination register index */ Reg16,
     ),
@@ -644,17 +653,17 @@ pub enum BytesOp {
     /// Check equality of two strings, putting result into `st0`.
     ///
     /// If both of strings are uninitialized, `st0` assigned `true` value.
-    #[display("cmp     s16[{0}],s16[{0}]")]
-    Eq(u8, u8),
+    #[display("cmp     {0},{1}")]
+    Eq(RegS, RegS),
 
     /// Compute offset and length of the `n`th fragment shared between two strings ("conjoint
     /// fragment"), putting it to the destination `u16` registers. If strings have no conjoint
     /// fragment sets destination to `None`.
-    #[display("con     s16[{0}],s16[{1}],u16{2},u16{3}")]
+    #[display("con     {0},{1},a16{2},u16{3},u16{4}")]
     Con(
-        /** First source string register */ u8,
-        /** Second source string register */ u8,
-        /** Number of the conjoint fragment to match */ u6,
+        /** First source string register */ RegS,
+        /** Second source string register */ RegS,
+        /** Index of the conjoint fragment to match */ Reg32,
         /** `u16` register index to save the offset of the conjoint fragment */ Reg32,
         /** `u16` register index to save the length of the conjoint fragment */ Reg32,
     ),
@@ -665,8 +674,8 @@ pub enum BytesOp {
     ///
     /// If the number of occurrences is `u16::MAX + 1`, sets `a16[1]` to `u16::MAX` and `st0` to
     /// `false`.
-    #[display("find    s16[{0}],s16[{1}],a16[1]")]
-    Find(/** `s` register with string */ u8, /** `s` register with matching fragment */ u8),
+    #[display("find    {0},{1},a16[1]")]
+    Find(/** `s` register with string */ RegS, /** `s` register with matching fragment */ RegS),
 
     /// Extract byte string slice into general `r` register. The length of the extracted string is
     /// equal to the bit dimension of the destination register. If the bit size of the destination
@@ -675,8 +684,8 @@ pub enum BytesOp {
     ///
     /// If the source string register - or offset register is uninitialized, sets destination to
     /// uninitialized state and `st0` to `false`.
-    #[display("extr    s16{0},a16{1},{2}{3}")]
-    Extr(/** `s` register index */ Reg32, RegR, Reg16, /** `a16` register with offset */ Reg16),
+    #[display("extr    {0},{1}{2},{1}{3}")]
+    Extr(/** `s` register index */ RegS, RegR, Reg16, /** `a16` register with offset */ Reg16),
 
     /// Inject general `R` register value at a given position to string register, replacing value
     /// of the corresponding bytes. If the insert offset is larger than the current length of the
@@ -684,9 +693,9 @@ pub enum BytesOp {
     /// are initialized with zeros. If the length of the inserted string plus insert offset exceeds
     /// the maximum string register length (2^16 bytes), than the destination register is set to
     /// `None` state and `st0` is set to `false`. Otherwise, `st0` value is not modified.
-    #[display("inj     s16{0},a16{1},{2}{3}")]
+    #[display("inj     {0},{1}{2},{1}{3}")]
     Inj(
-        /** `s` register index acting as the source and destination */ Reg32,
+        /** `s` register index acting as the source and destination */ RegS,
         RegR,
         Reg16,
         /** `a16` register with offset */ Reg16,
@@ -696,8 +705,8 @@ pub enum BytesOp {
     /// of the joined string exceeds the maximum string register length (2^16 bytes), than the
     /// destination register is set to `None` state and `st0` is set to `false`. Otherwise,
     /// `st0` value is not modified.
-    #[display("join    s16[{0}],s16[{1}],s16[{2}]")]
-    Join(/** Source 1 */ u8, /** Source 2 */ u8, /** Destination */ u8),
+    #[display("join    {0},{1},{2}")]
+    Join(/** Source 1 */ RegS, /** Source 2 */ RegS, /** Destination */ RegS),
 
     /// Split bytestring at a given offset taken from `a16` register into two destination strings,
     /// overwriting their value. If offset exceeds the length of the string in the register,
@@ -732,13 +741,13 @@ pub enum BytesOp {
     /// Rule on `st0` changes: if at least one of the destination registers is set to `None`, or
     /// `offset` value exceeds source string length, `st0` is set to `false`; otherwise its value
     /// is not modified
-    #[display("splt:{2}  s16[{0}],a16{1},s16[{3}],s16[{4}]")]
+    #[display("splt.{2}  {0},a16{1},{3},{4}")]
     Splt(
         SplitFlag,
         /** `a16` register index with offset value */ Reg32,
-        /** Source */ u8,
-        /** Destination 1 */ u8,
-        /** Destination 2 */ u8,
+        /** Source */ RegS,
+        /** Destination 1 */ RegS,
+        /** Destination 2 */ RegS,
     ),
 
     /// Insert value from one of bytestring register at a given index of other bytestring register,
@@ -770,12 +779,12 @@ pub enum BytesOp {
     /// </pre>
     ///
     /// In all of these cases `st0` is set to `false`. Otherwise, `st0` value is not modified.
-    #[display("ins:{3}   s16[{0}],s16[{1}],a16{2}")]
+    #[display("ins.{3}   {0},{1},a16{2}")]
     Ins(
         InsertFlag,
         /** `a16` register index with offset value for insert location */ Reg32,
-        /** Source register */ u8,
-        /** Destination register */ u8,
+        /** Source register */ RegS,
+        /** Destination register */ RegS,
     ),
 
     /// Delete bytes in a given range, shifting the remaining bytes leftward. The start offset is
@@ -796,7 +805,7 @@ pub enum BytesOp {
     /// `offset_start > src_len`:
     ///   (1) set destination to `None`
     ///   (2) set destination to zero-length string
-    /// `offset_end > src_len && offser_start <= src_len`:
+    /// `offset_end > src_len && offset_start <= src_len`:
     ///   (1) set destination to `None`
     ///   (3) set destination to the fragment of the string `offset_start..src_len`
     ///   (4) set destination to the fragment of the string `offset_start..src_len` and extend
@@ -806,7 +815,7 @@ pub enum BytesOp {
     /// `flag1` and `flag2` arguments indicate whether `st0` should be set to `false` if
     /// `offset_start > src_len` and `offset_end > src_len && offser_start <= src_len`.
     /// In all other cases, `st0` value is not modified.
-    #[display("del:{6}   s16[{0}],s16[{1}],{2}{3},{4}{5}")]
+    #[display("del.{0}   {7},{8},{1}{2},{3}{4},{5},{6}")]
     Del(
         DeleteFlag,
         RegA2,
@@ -817,16 +826,16 @@ pub enum BytesOp {
         /** `flag2` indicating `st0` value set to false if
          * `offset_end > src_len && offser_start <= src_len` */
         bool,
-        /** Source `s` register */ u8,
-        /** Destination `s` register */ u8,
+        /** Source `s` register */ RegS,
+        /** Destination `s` register */ RegS,
     ),
 
     /// Revert byte order of the string.
     ///
     /// If the source string register is uninitialized, resets destination to the uninitialized
     /// state and sets `st0` to `false`.
-    #[display("rev     s16[{0}],s16[{1}]")]
-    Rev(/** Source */ u8, /** Destination */ u8),
+    #[display("rev     {0},{1}")]
+    Rev(/** Source */ RegS, /** Destination */ RegS),
 }
 
 /// Cryptographic hashing functions
@@ -837,30 +846,30 @@ pub enum DigestOp {
     ///
     /// Sets `st0` to `false` and destination register to `None` if the source register does not
     /// contain a value
-    #[display("ripemd  s16{0},r160{1}")]
+    #[display("ripemd  {0},r160{1}")]
     Ripemd(
-        /** Index of string register */ Reg32,
-        /** Index of `r160` register to save result to */ Reg8,
+        /** Index of string register */ RegS,
+        /** Index of `r160` register to save result to */ Reg16,
     ),
 
     /// Computes SHA256 hash value
     ///
     /// Sets `st0` to `false` and destination register to `None` if the source register does not
     /// contain a value
-    #[display("sha2    s16{0},r256{1}")]
+    #[display("sha2    {0},r256{1}")]
     Sha256(
-        /** Index of string register */ Reg32,
-        /** Index of `r256` register to save result to */ Reg8,
+        /** Index of string register */ RegS,
+        /** Index of `r256` register to save result to */ Reg16,
     ),
 
     /// Computes SHA256 hash value
     ///
     /// Sets `st0` to `false` and destination register to `None` if the source register does not
     /// contain a value
-    #[display("sha2    s16{0},r512{1}")]
+    #[display("sha2    {0},r512{1}")]
     Sha512(
-        /** Index of string register */ Reg32,
-        /** Index of `r512` register to save result to */ Reg8,
+        /** Index of string register */ RegS,
+        /** Index of `r512` register to save result to */ Reg16,
     ),
 }
 
