@@ -13,12 +13,11 @@
 
 pub mod constants;
 mod cursor;
-mod library;
+mod model;
 mod rw;
+mod segs;
 
-pub(crate) use cursor::Cursor;
-pub use library::{
-    AssemblerError, Lib, LibId, LibIdTag, LibSeg, LibSegOverflow, LibSite, SegmentError,
-};
-pub use rw::{CodeEofError, WriteError};
-pub(crate) use rw::{Read, Write};
+pub use cursor::Cursor;
+pub use model::{AssemblerError, Lib, LibId, LibIdError, LibIdTag, LibSite};
+pub use rw::{CodeEofError, Read, Write, WriteError};
+pub use segs::{IsaSeg, IsaSegError, LibSeg, LibSegOverflow, SegmentError};
