@@ -10,7 +10,6 @@
 // If not, see <https://opensource.org/licenses/MIT>.
 
 use alloc::boxed::Box;
-use alloc::collections::BTreeMap;
 use alloc::string::ToString;
 use core::fmt::{self, Debug, Formatter};
 
@@ -18,7 +17,7 @@ use amplify::num::{u1024, u256, u512};
 use half::bf16;
 use rustc_apfloat::ieee;
 
-use super::{Reg32, RegA, RegAFR, RegF, RegR};
+use super::{Reg32, RegA, RegAFR, RegF, RegR, RegS};
 use crate::data::{ByteStr, MaybeNumber, Number};
 use crate::isa::InstructionSet;
 use crate::libs::LibSite;
@@ -39,7 +38,7 @@ pub struct CoreRegs {
     pub(crate) a128: [Option<u128>; 32],
     pub(crate) a256: [Option<u256>; 32],
     pub(crate) a512: [Option<u512>; 32],
-    pub(crate) a1024: [Option<u1024>; 32],
+    pub(crate) a1024: Box<[Option<u1024>; 32]>,
 
     // Arithmetic float registers
     pub(crate) f16b: [Option<bf16>; 32],
@@ -58,13 +57,13 @@ pub struct CoreRegs {
     pub(crate) r160: [Option<[u8; 20]>; 32],
     pub(crate) r256: [Option<[u8; 32]>; 32],
     pub(crate) r512: [Option<[u8; 64]>; 32],
-    pub(crate) r1024: [Option<[u8; 128]>; 32],
-    pub(crate) r2048: [Option<[u8; 256]>; 32],
-    pub(crate) r4096: [Option<[u8; 512]>; 32],
-    pub(crate) r8192: [Option<[u8; 1024]>; 32],
+    pub(crate) r1024: Box<[Option<[u8; 128]>; 32]>,
+    pub(crate) r2048: Box<[Option<[u8; 256]>; 32]>,
+    pub(crate) r4096: Box<[Option<[u8; 512]>; 32]>,
+    pub(crate) r8192: Box<[Option<[u8; 1024]>; 32]>,
 
     /// String and bytestring registers
-    pub(crate) s16: BTreeMap<u8, ByteStr>,
+    pub(crate) s16: Box<[Option<ByteStr>; 16]>,
 
     /// Control flow register which stores result of equality, comparison, boolean check and
     /// overflowing operations. Initialized with `true`.
@@ -83,13 +82,13 @@ pub struct CoreRegs {
     ///
     /// - [`CoreRegs::cy0`] register
     /// - [`CoreRegs::cl0`] register
-    ca0: u32,
+    ca0: u64,
 
     /// Complexity limit
     ///
     /// If this register has a value set, once [`CoreRegs::ca0`] will reach this value the VM will
     /// stop program execution setting `st0` to `false`.
-    cl0: Option<u32>,
+    cl0: Option<u64>,
 
     /// Call stack
     ///
@@ -236,7 +235,9 @@ impl CoreRegs {
 
     /// Returns value from one of `S`-registers
     #[inline]
-    pub fn get_s(&self, index: impl Into<u8>) -> Option<&ByteStr> { self.s16.get(&index.into()) }
+    pub fn get_s(&self, index: impl Into<RegS>) -> Option<&ByteStr> {
+        self.s16[index.into().as_usize()].as_ref()
+    }
 
     /// Returns value from two registers only if both of them contain a value; otherwise returns
     /// `None`.
@@ -254,7 +255,11 @@ impl CoreRegs {
     /// Returns value from two string (`S`) registers only if both of them contain a value;
     /// otherwise returns `None`.
     #[inline]
-    pub fn get_both_s(&self, idx1: u8, idx2: u8) -> Option<(&ByteStr, &ByteStr)> {
+    pub fn get_both_s(
+        &self,
+        idx1: impl Into<RegS>,
+        idx2: impl Into<RegS>,
+    ) -> Option<(&ByteStr, &ByteStr)> {
         self.get_s(idx1).and_then(|val1| self.get_s(idx2).map(|val2| (val1, val2)))
     }
 
