@@ -9,12 +9,15 @@
 // You should have received a copy of the MIT License along with this software.
 // If not, see <https://opensource.org/licenses/MIT>.
 
+use core::convert::TryFrom;
+
 use amplify::num::{u1, u3, u4};
 
 use crate::data as number;
+use crate::reg::Register;
 
 /// Common set of methods handled by different sets and families of VM registers
-pub trait RegisterFamily {
+pub trait NumericRegister: Register {
     /// Register bit dimension
     #[inline]
     fn bits(&self) -> u16 { self.bytes() * 8 }
@@ -63,7 +66,16 @@ pub enum RegA {
     A1024 = 7,
 }
 
-impl RegisterFamily for RegA {
+impl Default for RegA {
+    fn default() -> Self { RegA::A64 }
+}
+
+impl Register for RegA {
+    #[inline]
+    fn description() -> &'static str { "A register" }
+}
+
+impl NumericRegister for RegA {
     #[inline]
     fn bytes(&self) -> u16 {
         match self {
@@ -147,6 +159,13 @@ impl From<&RegA2> for RegA {
     }
 }
 
+impl TryFrom<RegAll> for RegA {
+    type Error = ();
+
+    #[inline]
+    fn try_from(value: RegAll) -> Result<Self, Self::Error> { value.reg_a().ok_or(()) }
+}
+
 /// Enumeration of integer arithmetic registers suited for string addresses (`a8` and `a16`
 /// registers)
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
@@ -161,7 +180,16 @@ pub enum RegA2 {
     A16 = 1,
 }
 
-impl RegisterFamily for RegA2 {
+impl Default for RegA2 {
+    fn default() -> Self { RegA2::A8 }
+}
+
+impl Register for RegA2 {
+    #[inline]
+    fn description() -> &'static str { "A8 or A16 register" }
+}
+
+impl NumericRegister for RegA2 {
     #[inline]
     fn bytes(&self) -> u16 {
         match self {
@@ -203,6 +231,19 @@ impl From<u1> for RegA2 {
     }
 }
 
+impl TryFrom<RegAll> for RegA2 {
+    type Error = ();
+
+    #[inline]
+    fn try_from(value: RegAll) -> Result<Self, Self::Error> {
+        match value.reg_a() {
+            Some(RegA::A8) => Ok(RegA2::A8),
+            Some(RegA::A16) => Ok(RegA2::A16),
+            _ => Err(()),
+        }
+    }
+}
+
 /// Enumeration of float arithmetic registers (`F`-registers)
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
 #[repr(u8)]
@@ -240,7 +281,16 @@ pub enum RegF {
     F512 = 7,
 }
 
-impl RegisterFamily for RegF {
+impl Default for RegF {
+    fn default() -> Self { RegF::F64 }
+}
+
+impl Register for RegF {
+    #[inline]
+    fn description() -> &'static str { "F register" }
+}
+
+impl NumericRegister for RegF {
     #[inline]
     fn bytes(&self) -> u16 {
         match self {
@@ -317,6 +367,13 @@ impl From<u3> for RegF {
     }
 }
 
+impl TryFrom<RegAll> for RegF {
+    type Error = ();
+
+    #[inline]
+    fn try_from(value: RegAll) -> Result<Self, Self::Error> { value.reg_f().ok_or(()) }
+}
+
 /// Enumeration of the set of general registers (`R`-registers: non-arithmetic registers, mostly
 /// used for cryptography)
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
@@ -355,7 +412,16 @@ pub enum RegR {
     R8192 = 7,
 }
 
-impl RegisterFamily for RegR {
+impl Default for RegR {
+    fn default() -> Self { RegR::R256 }
+}
+
+impl Register for RegR {
+    #[inline]
+    fn description() -> &'static str { "R register" }
+}
+
+impl NumericRegister for RegR {
     #[inline]
     fn bytes(&self) -> u16 {
         match self {
@@ -416,6 +482,148 @@ impl From<u3> for RegR {
     }
 }
 
+impl TryFrom<RegAll> for RegR {
+    type Error = ();
+
+    #[inline]
+    fn try_from(value: RegAll) -> Result<Self, Self::Error> { value.reg_r().ok_or(()) }
+}
+
+/// Superset of all registers accessible via intstructions. The superset includes `A`, `F`, `R` and
+/// `S` families of registers.
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display, From)]
+#[display(inner)]
+pub enum RegAll {
+    /// Arithmetic integer registers (`A` registers)
+    #[from]
+    A(RegA),
+
+    /// Arithmetic float registers (`F` registers)
+    #[from]
+    F(RegF),
+
+    /// Non-arithmetic (general) registers (`R` registers)
+    #[from]
+    R(RegR),
+
+    /// String registers (`S` registers)
+    S,
+}
+
+impl Default for RegAll {
+    fn default() -> Self { RegAll::A(Default::default()) }
+}
+
+impl Register for RegAll {
+    #[inline]
+    fn description() -> &'static str { "A, F, R or S register" }
+}
+
+impl RegAll {
+    /// Returns inner A-register type, if any
+    #[inline]
+    pub fn reg_a(self) -> Option<RegA> {
+        match self {
+            RegAll::A(a) => Some(a),
+            _ => None,
+        }
+    }
+
+    /// Returns inner F-register type, if any
+    #[inline]
+    pub fn reg_f(self) -> Option<RegF> {
+        match self {
+            RegAll::F(f) => Some(f),
+            _ => None,
+        }
+    }
+
+    /// Returns inner R-register type, if any
+    #[inline]
+    pub fn reg_r(self) -> Option<RegR> {
+        match self {
+            RegAll::R(r) => Some(r),
+            _ => None,
+        }
+    }
+
+    /// Returns string describing the family of the register
+    #[inline]
+    pub fn family_name(self) -> &'static str {
+        match self {
+            RegAll::A(_) => RegA::description(),
+            RegAll::F(_) => RegF::description(),
+            RegAll::R(_) => RegR::description(),
+            RegAll::S => "S register",
+        }
+    }
+}
+
+impl From<&RegA> for RegAll {
+    #[inline]
+    fn from(reg: &RegA) -> Self { Self::A(*reg) }
+}
+
+impl From<&RegF> for RegAll {
+    #[inline]
+    fn from(reg: &RegF) -> Self { Self::F(*reg) }
+}
+
+impl From<&RegR> for RegAll {
+    #[inline]
+    fn from(reg: &RegR) -> Self { Self::R(*reg) }
+}
+
+impl From<RegA2> for RegAll {
+    #[inline]
+    fn from(reg: RegA2) -> Self { Self::A(reg.into()) }
+}
+
+impl From<&RegA2> for RegAll {
+    #[inline]
+    fn from(reg: &RegA2) -> Self { Self::A(reg.into()) }
+}
+
+impl From<RegAF> for RegAll {
+    #[inline]
+    fn from(reg: RegAF) -> Self {
+        match reg {
+            RegAF::A(a) => Self::A(a),
+            RegAF::F(f) => Self::F(f),
+        }
+    }
+}
+
+impl From<&RegAF> for RegAll {
+    #[inline]
+    fn from(reg: &RegAF) -> Self {
+        match reg {
+            RegAF::A(a) => Self::A(*a),
+            RegAF::F(f) => Self::F(*f),
+        }
+    }
+}
+
+impl From<RegAR> for RegAll {
+    #[inline]
+    fn from(reg: RegAR) -> Self {
+        match reg {
+            RegAR::A(a) => Self::A(a),
+            RegAR::R(r) => Self::R(r),
+        }
+    }
+}
+
+impl From<&RegAR> for RegAll {
+    #[inline]
+    fn from(reg: &RegAR) -> Self {
+        match reg {
+            RegAR::A(a) => Self::A(*a),
+            RegAR::R(r) => Self::R(*r),
+        }
+    }
+}
+
 /// Superset of all registers which value can be represented by a
 /// [`crate::data::Number`]/[`crate::data::MaybeNumber`]. The superset includes `A`, `F`, and
 /// `R` families of registers.
@@ -435,7 +643,16 @@ pub enum RegAFR {
     R(RegR),
 }
 
-impl RegisterFamily for RegAFR {
+impl Default for RegAFR {
+    fn default() -> Self { RegAFR::A(Default::default()) }
+}
+
+impl Register for RegAFR {
+    #[inline]
+    fn description() -> &'static str { "A, F or R register" }
+}
+
+impl NumericRegister for RegAFR {
     #[inline]
     fn bytes(&self) -> u16 {
         match self {
@@ -549,6 +766,20 @@ impl From<&RegAR> for RegAFR {
     }
 }
 
+impl TryFrom<RegAll> for RegAFR {
+    type Error = ();
+
+    #[inline]
+    fn try_from(value: RegAll) -> Result<Self, Self::Error> {
+        match value {
+            RegAll::A(a) => Ok(RegAFR::A(a)),
+            RegAll::F(f) => Ok(RegAFR::F(f)),
+            RegAll::R(r) => Ok(RegAFR::R(r)),
+            _ => Err(()),
+        }
+    }
+}
+
 /// Superset of `A` and `F` arithmetic registers
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display, From)]
 #[display(inner)]
@@ -562,7 +793,16 @@ pub enum RegAF {
     F(RegF),
 }
 
-impl RegisterFamily for RegAF {
+impl Default for RegAF {
+    fn default() -> Self { RegAF::A(Default::default()) }
+}
+
+impl Register for RegAF {
+    #[inline]
+    fn description() -> &'static str { "A or F register" }
+}
+
+impl NumericRegister for RegAF {
     #[inline]
     fn bytes(&self) -> u16 {
         match self {
@@ -632,6 +872,19 @@ impl From<&RegA2> for RegAF {
     fn from(reg: &RegA2) -> Self { Self::A(reg.into()) }
 }
 
+impl TryFrom<RegAll> for RegAF {
+    type Error = ();
+
+    #[inline]
+    fn try_from(value: RegAll) -> Result<Self, Self::Error> {
+        match value {
+            RegAll::A(a) => Ok(RegAF::A(a)),
+            RegAll::F(f) => Ok(RegAF::F(f)),
+            _ => Err(()),
+        }
+    }
+}
+
 /// Superset of `A` and `R` registers
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display, From)]
 #[display(inner)]
@@ -645,7 +898,16 @@ pub enum RegAR {
     R(RegR),
 }
 
-impl RegisterFamily for RegAR {
+impl Default for RegAR {
+    fn default() -> Self { RegAR::A(Default::default()) }
+}
+
+impl Register for RegAR {
+    #[inline]
+    fn description() -> &'static str { "A or R register" }
+}
+
+impl NumericRegister for RegAR {
     #[inline]
     fn bytes(&self) -> u16 {
         match self {
@@ -725,6 +987,19 @@ impl From<&RegA2> for RegAR {
     fn from(reg: &RegA2) -> Self { Self::A(reg.into()) }
 }
 
+impl TryFrom<RegAll> for RegAR {
+    type Error = ();
+
+    #[inline]
+    fn try_from(value: RegAll) -> Result<Self, Self::Error> {
+        match value {
+            RegAll::A(a) => Ok(RegAR::A(a)),
+            RegAll::R(r) => Ok(RegAR::R(r)),
+            _ => Err(()),
+        }
+    }
+}
+
 /// Block of registers, either integer arithmetic or non-arithmetic (general) registers
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
 pub enum RegBlockAR {
@@ -737,6 +1012,15 @@ pub enum RegBlockAR {
     R,
 }
 
+impl Default for RegBlockAR {
+    fn default() -> Self { RegBlockAR::A }
+}
+
+impl Register for RegBlockAR {
+    #[inline]
+    fn description() -> &'static str { "A or R register block" }
+}
+
 impl RegBlockAR {
     /// Converts value into specific register matching the provided bit dimension. If the register
     /// with the given dimension does not exists, returns `None`.
@@ -748,6 +1032,19 @@ impl RegBlockAR {
     }
 }
 
+impl TryFrom<RegAll> for RegBlockAR {
+    type Error = ();
+
+    fn try_from(value: RegAll) -> Result<Self, Self::Error> {
+        match value {
+            RegAll::A(_) => Ok(RegBlockAR::A),
+            RegAll::F(_) => Err(()),
+            RegAll::R(_) => Ok(RegBlockAR::R),
+            RegAll::S => Err(()),
+        }
+    }
+}
+
 /// Block of registers, either integer, float arithmetic or non-arithmetic (general) registers
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
 pub enum RegBlockAFR {
@@ -764,6 +1061,15 @@ pub enum RegBlockAFR {
     R,
 }
 
+impl Default for RegBlockAFR {
+    fn default() -> Self { RegBlockAFR::A }
+}
+
+impl Register for RegBlockAFR {
+    #[inline]
+    fn description() -> &'static str { "A, F or R register block" }
+}
+
 impl RegBlockAFR {
     /// Converts value into specific register matching the provided bit dimension. If the register
     /// with the given dimension does not exists, returns `None`.
@@ -775,3 +1081,56 @@ impl RegBlockAFR {
         }
     }
 }
+
+impl TryFrom<RegAll> for RegBlockAFR {
+    type Error = ();
+
+    fn try_from(value: RegAll) -> Result<Self, Self::Error> {
+        match value {
+            RegAll::A(_) => Ok(RegBlockAFR::A),
+            RegAll::F(_) => Ok(RegBlockAFR::F),
+            RegAll::R(_) => Ok(RegBlockAFR::R),
+            RegAll::S => Err(()),
+        }
+    }
+}
+
+/// Blocks of registers including all non-control register types
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Display)]
+pub enum RegBlock {
+    /// Arithmetic integer registers (`A` registers)
+    #[display("a")]
+    A,
+
+    /// Arithmetic float registers (`F` registers)
+    #[display("f")]
+    F,
+
+    /// Non-arithmetic (generic) registers (`R` registers)
+    #[display("r")]
+    R,
+
+    /// Byte-string registers (`S` registers)
+    #[display("s")]
+    S,
+}
+
+impl Default for RegBlock {
+    fn default() -> Self { RegBlock::A }
+}
+
+impl Register for RegBlock {
+    #[inline]
+    fn description() -> &'static str { "A, F, R or S register block" }
+}
+
+impl From<RegAll> for RegBlock {
+    fn from(reg: RegAll) -> Self {
+        match reg {
+            RegAll::A(_) => RegBlock::A,
+            RegAll::F(_) => RegBlock::F,
+            RegAll::R(_) => RegBlock::R,
+            RegAll::S => RegBlock::S,
+        }
+    }
+}
